@@ -99,6 +99,7 @@ col_init(struct col_t *pcol, int sybtype, int collen)
 	if (pcol->type == TDS_INVALID_TYPE)
 		return NULL;
 	pcol->len = collen;
+	pcol->s = NULL;
 
 	switch (sybtype) {
 	case 0:
@@ -143,10 +144,14 @@ col_equal(const struct col_t *pc1, const struct col_t *pc2)
 			return false;
 		return strncmp(pc1->s, pc2->s, pc1->len) == 0;
 	case SYBINT1:
+	case SYBUINT1:
+	case SYBSINT1:
 		return pc1->data.ti == pc2->data.ti;
 	case SYBINT2:
+	case SYBUINT2:
 		return pc1->data.si == pc2->data.si;
 	case SYBINT4:
+	case SYBUINT4:
 		return pc1->data.i == pc2->data.i;
 	case SYBFLT8:
 		return pc1->data.f == pc2->data.f;
@@ -172,6 +177,36 @@ col_equal(const struct col_t *pc1, const struct col_t *pc2)
 	case SYBMONEYN:
 	case SYBDATETIMN:
 	case SYBMSTABLE:
+	case SYBNVARCHAR:
+	case SYBINT8:
+	case XSYBCHAR:
+	case XSYBVARCHAR:
+	case XSYBNVARCHAR:
+	case XSYBNCHAR:
+	case XSYBVARBINARY:
+	case XSYBBINARY:
+	case SYBUNIQUE:
+	case SYBVARIANT:
+	case SYBMSUDT:
+	case SYBMSXML:
+	case SYBMSDATE:
+	case SYBMSTIME:
+	case SYBMSDATETIME2:
+	case SYBMSDATETIMEOFFSET:
+	case SYBLONGBINARY:
+	case SYBUINT8:
+	case SYBDATE:
+	case SYBDATEN:
+	case SYB5INT8:
+	case SYBINTERVAL:
+	case SYBTIME:
+	case SYBTIMEN:
+	case SYBUINTN:
+	case SYBUNITEXT:
+	case SYBXML:
+	case SYB5BIGDATETIME:
+	case SYB5BIGTIME:
+
 		assert( false && pc1->type );
 		break;
 	}
@@ -187,10 +222,14 @@ col_buffer(struct col_t *pcol)
 	case SYBVARCHAR:
 		return pcol->s;
 	case SYBINT1:
+	case SYBUINT1:
+	case SYBSINT1:
 		return &pcol->data.ti;
 	case SYBINT2:
+	case SYBUINT2:
 		return &pcol->data.si;
 	case SYBINT4:
+	case SYBUINT4:
 		return &pcol->data.i;
 	case SYBFLT8:
 		return &pcol->data.f;
@@ -216,6 +255,35 @@ col_buffer(struct col_t *pcol)
 	case SYBMONEYN:
 	case SYBDATETIMN:
 	case SYBMSTABLE:
+	case SYBNVARCHAR:
+	case SYBINT8:
+	case XSYBCHAR:
+	case XSYBVARCHAR:
+	case XSYBNVARCHAR:
+	case XSYBNCHAR:
+	case XSYBVARBINARY:
+	case XSYBBINARY:
+	case SYBUNIQUE:
+	case SYBVARIANT:
+	case SYBMSUDT:
+	case SYBMSXML:
+	case SYBMSDATE:
+	case SYBMSTIME:
+	case SYBMSDATETIME2:
+	case SYBMSDATETIMEOFFSET:
+	case SYBLONGBINARY:
+	case SYBUINT8:
+	case SYBDATE:
+	case SYBDATEN:
+	case SYB5INT8:
+	case SYBINTERVAL:
+	case SYBTIME:
+	case SYBTIMEN:
+	case SYBUINTN:
+	case SYBUNITEXT:
+	case SYBXML:
+	case SYB5BIGDATETIME:
+	case SYB5BIGTIME:
 		assert( false && pcol->type );
 		break;
 	}
@@ -529,7 +597,7 @@ make_col_name(DBPROCESS *dbproc, const KEY_T *k)
 		return NULL;
 	}
 	for(pc=k->keys; pc < k->keys + k->nkeys; pc++) {
-		*s++ = strdup(string_value(pc));
+		*s++ = string_value(pc);
 	}
 	
 	output = join(k->nkeys, names, "/");
@@ -841,7 +909,7 @@ dbnextrow_pivoted(DBPROCESS *dbproc, PIVOT_T *pp)
 			}
 		}
 		if (!pcol->column_varaddr) {
-			fprintf(stderr, "no pcol->column_varaddr in col %d\n", i);
+			tdsdump_log(TDS_DBG_ERROR, "no pcol->column_varaddr in col %d\n", i);
 			continue;
 		}
 
@@ -866,16 +934,6 @@ dbnextrow_pivoted(DBPROCESS *dbproc, PIVOT_T *pp)
 		
 		assert(pval);
 		
-#if 0
-		printf("\ncopying col %d, type %d/%d, len %d to %p ", i, pval->type, pcol->column_type, pval->len, pcol->column_varaddr);
-		switch (pval->type) {
-		case 48:
-			printf("value %d", (int)pval->ti);	break;
-		case 56:
-			printf("value %d", (int)pval->si);	break;
-		}
-		printf("\n");
-#endif		
 		pcol->column_size = pval->len;
 		pcol->column_data = col_buffer(pval);
 		
@@ -924,7 +982,7 @@ dbpivot(DBPROCESS *dbproc, int nkeys, int *keys, int ncols, int *cols, DBPIVOT_F
 	tdsdump_log(TDS_DBG_FUNC, "dbpivot(%p, %d,%p, %d,%p, %p, %d)\n", dbproc, nkeys, keys, ncols, cols, func, val);
 	if (logalot) {
 		char buffer[1024] = {'\0'}, *s = buffer;
-		const static char *const names[2] = { "\tkeys (down)", "\n\tcols (across)" };
+		static const char *const names[2] = { "\tkeys (down)", "\n\tcols (across)" };
 		int *p = keys, *pend = p + nkeys;
 		
 		for (i=0; i < 2; i++) {
@@ -1074,20 +1132,6 @@ dbpivot(DBPROCESS *dbproc, int nkeys, int *keys, int ncols, int *cols, DBPIVOT_F
 	}
 	
 	return SUCCEED;
-	
-#if 0
-	for (pp->pout=pp->output; pp->pout < pp->output + pp->nout; pp->pout++) {
-		char name[256] = {0};
-	
-		assert(pp->pout->col_key.keys[0].len < sizeof(name));
-		memset(name, '\0', sizeof(name));
-		memcpy(name, pp->pout->col_key.keys[0].s, pp->pout->col_key.keys[0].len), 
-		printf("%5d  %-30s  %5d\n", pp->pout->row_key.keys[0].i, 
-					    name, 
-					    pp->pout->value.i );
-	}
-	exit(1);
-#endif
 }
 
 /* 
