@@ -1,6 +1,6 @@
 /* FreeTDS - Library of routines accessing Sybase and Microsoft databases
  * Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005  Brian Bruns
- * Copyright (C) 2005-2011  Frediano Ziglio
+ * Copyright (C) 2005-2024  Frediano Ziglio
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -29,10 +29,11 @@
 
 #include <freetds/odbc.h>
 #include <freetds/utils/string.h>
+#include <freetds/utils/path.h>
 #include <freetds/utils.h>
 #include <freetds/replacements.h>
 
-#define ODBC_PARAM(p) static const char odbc_param_##p[] = #p;
+#define ODBC_PARAM(p) const char odbc_param_##p[] = #p;
 ODBC_PARAM_LIST
 #undef ODBC_PARAM
 
@@ -94,8 +95,11 @@ static FILE *tdoGetIniFileName(void);
  *    expect see tdoGetIniFileName().
  *
  */
-static int SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault, LPSTR pRetBuffer, int nRetBuffer,
-				      LPCSTR pszFileName);
+#ifndef _WIN32
+static
+#endif
+int SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault,
+			       LPSTR pRetBuffer, int nRetBuffer, LPCSTR pszFileName);
 #endif
 
 #if defined(FILENAME_MAX) && FILENAME_MAX < 512
@@ -138,57 +142,80 @@ myGetPrivateProfileString(const char *DSN, const char *key, char *buf)
 	return SQLGetPrivateProfileString(DSN, key, "", buf, FILENAME_MAX, "odbc.ini");
 }
 
+/**
+ * Converts Encrypt option to Encryption option.
+ * Encryption is FreeTDS specific and older than Encrypt.
+ */
+static const char *
+odbc_encrypt2encryption(const char *encrypt)
+{
+	if (strcasecmp(encrypt, "strict") == 0)
+		return TDS_STR_ENCRYPTION_STRICT;
+
+	if (strcasecmp(encrypt, "mandatory") == 0
+	    || strcasecmp(encrypt, "true") == 0
+	    || strcasecmp(encrypt, "yes") == 0)
+		return TDS_STR_ENCRYPTION_REQUIRE;
+
+	if (strcasecmp(encrypt, "optional") == 0
+	    || strcasecmp(encrypt, "false") == 0
+	    || strcasecmp(encrypt, "no") == 0)
+		return TDS_STR_ENCRYPTION_REQUEST;
+
+	return "invalid_encrypt";
+}
+
 /** 
  * Read connection information from given DSN
  * @param DSN           DSN name
  * @param login    where to store connection info
- * @return 1 if success 0 otherwhise
+ * @return true if success false otherwhise
  */
-int
+bool
 odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 {
 	char tmp[FILENAME_MAX];
-	int freetds_conf_less = 1;
+	bool freetds_conf_less = true;
 
 	/* use old servername */
 	if (myGetPrivateProfileString(DSN, odbc_param_Servername, tmp) > 0) {
-		freetds_conf_less = 0;
+		freetds_conf_less = false;
 		if (!tds_dstr_copy(&login->server_name, tmp)) {
 			odbc_errs_add(errs, "HY001", NULL);
-			return 0;
+			return false;
 		}
 		tds_read_conf_file(login, tmp);
 		if (myGetPrivateProfileString(DSN, odbc_param_Server, tmp) > 0) {
 			odbc_errs_add(errs, "HY000", "You cannot specify both SERVERNAME and SERVER");
-			return 0;
+			return false;
 		}
 		if (myGetPrivateProfileString(DSN, odbc_param_Address, tmp) > 0) {
 			odbc_errs_add(errs, "HY000", "You cannot specify both SERVERNAME and ADDRESS");
-			return 0;
+			return false;
 		}
 	}
 
 	/* search for server (compatible with ms one) */
 	if (freetds_conf_less) {
-		int address_specified = 0;
+		bool address_specified = false;
 
 		if (myGetPrivateProfileString(DSN, odbc_param_Address, tmp) > 0) {
-			address_specified = 1;
+			address_specified = true;
 			/* TODO parse like MS */
 
 			if (TDS_FAILED(tds_lookup_host_set(tmp, &login->ip_addrs))) {
 				odbc_errs_add(errs, "HY000", "Error parsing ADDRESS attribute");
-				return 0;
+				return false;
 			}
 		}
 		if (myGetPrivateProfileString(DSN, odbc_param_Server, tmp) > 0) {
 			if (!tds_dstr_copy(&login->server_name, tmp)) {
 				odbc_errs_add(errs, "HY001", NULL);
-				return 0;
+				return false;
 			}
 			if (!address_specified) {
 				if (!parse_server(errs, tmp, login))
-					return 0;
+					return false;
 			}
 		}
 	}
@@ -206,7 +233,7 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 	    && myGetPrivateProfileString(DSN, odbc_param_Database, tmp) > 0)
 		if (!tds_dstr_copy(&login->database, tmp)) {
 			odbc_errs_add(errs, "HY001", NULL);
-			return 0;
+			return false;
 		}
 
 	if (myGetPrivateProfileString(DSN, odbc_param_TextSize, tmp) > 0)
@@ -230,6 +257,9 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 	if (myGetPrivateProfileString(DSN, odbc_param_Encryption, tmp) > 0)
 		tds_parse_conf_section(TDS_STR_ENCRYPTION, tmp, login);
 
+	if (myGetPrivateProfileString(DSN, odbc_param_Encrypt, tmp) > 0)
+		tds_parse_conf_section(TDS_STR_ENCRYPTION, odbc_encrypt2encryption(tmp), login);
+
 	if (myGetPrivateProfileString(DSN, odbc_param_UseNTLMv2, tmp) > 0)
 		tds_parse_conf_section(TDS_STR_USENTLMV2, tmp, login);
 
@@ -239,12 +269,14 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 	if (myGetPrivateProfileString(DSN, odbc_param_ServerSPN, tmp) > 0)
 		tds_parse_conf_section(TDS_STR_SPN, tmp, login);
 
-	if (myGetPrivateProfileString(DSN, odbc_param_Trusted_Connection, tmp) > 0 && tds_config_boolean(odbc_param_Trusted_Connection, tmp, login)) {
+	if (myGetPrivateProfileString(DSN, odbc_param_Trusted_Connection, tmp) > 0
+	    && tds_config_boolean(odbc_param_Trusted_Connection, tmp, login)) {
 		tds_dstr_empty(&login->user_name);
 		tds_dstr_empty(&login->password);
 	}
 
-	if (myGetPrivateProfileString(DSN, odbc_param_MARS_Connection, tmp) > 0 && tds_config_boolean(odbc_param_MARS_Connection, tmp, login)) {
+	if (myGetPrivateProfileString(DSN, odbc_param_MARS_Connection, tmp) > 0
+	    && tds_config_boolean(odbc_param_MARS_Connection, tmp, login)) {
 		login->mars = 1;
 	}
 
@@ -254,7 +286,15 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 	if (myGetPrivateProfileString(DSN, odbc_param_Timeout, tmp) > 0)
 		tds_parse_conf_section(TDS_STR_TIMEOUT, tmp, login);
 
-	return 1;
+	if (myGetPrivateProfileString(DSN, odbc_param_HostNameInCertificate, tmp) > 0
+	    && (tmp[0] && strcmp(tmp, "null") != 0)) {
+		if (!tds_dstr_copy(&login->certificate_host_name, tmp)) {
+			odbc_errs_add(errs, "HY001", NULL);
+			return false;
+		}
+	}
+
+	return true;
 }
 
 /**
@@ -340,7 +380,7 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 {
 	const char *p, *end;
 	DSTR *dest_s, value = DSTR_INITIALIZER;
-	enum { CFG_DSN = 1, CFG_SERVER = 2, CFG_SERVERNAME = 4 };
+	enum { CFG_DSN = 1, CFG_SERVERNAME = 2 };
 	unsigned int cfgs = 0;	/* flags for indicate second parse of string */
 	char option[24];
 	int trusted = 0;
@@ -381,20 +421,12 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 
 #define CHK_PARAM(p) (strcasecmp(option, odbc_param_##p) == 0 && (num_param=ODBC_PARAM_##p) >= 0)
 		if (CHK_PARAM(Server)) {
-			/* error if servername or DSN specified */
-			if ((cfgs & (CFG_DSN|CFG_SERVERNAME)) != 0) {
-				odbc_errs_add(errs, "HY000", "Only one between SERVER, SERVERNAME and DSN can be specified");
+			dest_s = &login->server_name;
+			if (!parse_server(errs, tds_dstr_buf(&value), login))
 				goto Cleanup;
-			}
-			if (!cfgs) {
-				dest_s = &login->server_name;
-				if (!parse_server(errs, tds_dstr_buf(&value), login))
-					goto Cleanup;
-				cfgs = CFG_SERVER;
-			}
 		} else if (CHK_PARAM(Servername)) {
-			if ((cfgs & (CFG_DSN|CFG_SERVER)) != 0) {
-				odbc_errs_add(errs, "HY000", "Only one between SERVER, SERVERNAME and DSN can be specified");
+			if ((cfgs & CFG_DSN) != 0) {
+				odbc_errs_add(errs, "HY000", "Only one between SERVERNAME and DSN can be specified");
 				goto Cleanup;
 			}
 			if (!cfgs) {
@@ -405,8 +437,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 				continue;
 			}
 		} else if (CHK_PARAM(DSN)) {
-			if ((cfgs & (CFG_SERVER|CFG_SERVERNAME)) != 0) {
-				odbc_errs_add(errs, "HY000", "Only one between SERVER, SERVERNAME and DSN can be specified");
+			if ((cfgs & CFG_SERVERNAME) != 0) {
+				odbc_errs_add(errs, "HY000", "Only one between SERVERNAME and DSN can be specified");
 				goto Cleanup;
 			}
 			if (!cfgs) {
@@ -448,6 +480,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 			tds_parse_conf_section(TDS_STR_DEBUGFLAGS, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(Encryption)) {
 			tds_parse_conf_section(TDS_STR_ENCRYPTION, tds_dstr_cstr(&value), login);
+		} else if (CHK_PARAM(Encrypt)) {
+			tds_parse_conf_section(TDS_STR_ENCRYPTION, odbc_encrypt2encryption(tds_dstr_cstr(&value)), login);
 		} else if (CHK_PARAM(UseNTLMv2)) {
 			tds_parse_conf_section(TDS_STR_USENTLMV2, tds_dstr_cstr(&value), login);
 		} else if (CHK_PARAM(REALM)) {
@@ -480,6 +514,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 			tdsdump_log(TDS_DBG_INFO1, "Application Intent %s\n", readonly_intent);
 		} else if (CHK_PARAM(Timeout)) {
 			tds_parse_conf_section(TDS_STR_TIMEOUT, tds_dstr_cstr(&value), login);
+		} else if (CHK_PARAM(HostNameInCertificate)) {
+			dest_s = &login->certificate_host_name;
 		}
 
 		if (num_param >= 0 && parsed_params) {
@@ -551,7 +587,7 @@ odbc_build_connect_string(TDS_ERRS *errs, TDS_PARSED_PARAM *params, char **out)
 
 #if !HAVE_SQLGETPRIVATEPROFILESTRING
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(TDS_NO_DM)
 #  error There is something wrong  in configuration...
 #endif
 
@@ -565,7 +601,7 @@ typedef struct
 }
 ProfileParam;
 
-static void
+static bool
 tdoParseProfile(const char *option, const char *value, void *param)
 {
 	ProfileParam *p = (ProfileParam *) param;
@@ -576,9 +612,13 @@ tdoParseProfile(const char *option, const char *value, void *param)
 		p->ret_val = strlen(p->buffer);
 		p->found = 1;
 	}
+	return true;
 }
 
-static int
+#ifndef _WIN32
+static
+#endif
+int
 SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault, LPSTR pRetBuffer, int nRetBuffer,
 			   LPCSTR pszFileName)
 {
@@ -633,7 +673,7 @@ SQLGetPrivateProfileString(LPCSTR pszSection, LPCSTR pszEntry, LPCSTR pszDefault
 }
 
 static FILE *
-tdoGetIniFileName()
+tdoGetIniFileName(void)
 {
 	FILE *ret = NULL;
 	char *p;
@@ -667,239 +707,3 @@ tdoGetIniFileName()
 }
 
 #endif /* !HAVE_SQLGETPRIVATEPROFILESTRING */
-
-#ifdef UNIXODBC
-
-/*
- * Begin BIG Hack.
- *  
- * We need these from odbcinstext.h but it wants to 
- * include <log.h> and <ini.h>, which are not in the 
- * standard include path.  XXX smurph
- * confirmed by unixODBC stuff, odbcinstext.h shouldn't be installed. freddy77
- */
-#define     INI_MAX_LINE            1000
-#define     INI_MAX_OBJECT_NAME     INI_MAX_LINE
-#define     INI_MAX_PROPERTY_NAME   INI_MAX_LINE
-#define     INI_MAX_PROPERTY_VALUE  INI_MAX_LINE
-
-#define	ODBCINST_PROMPTTYPE_LABEL		0	/* readonly */
-#define	ODBCINST_PROMPTTYPE_TEXTEDIT	1
-#define	ODBCINST_PROMPTTYPE_LISTBOX		2
-#define	ODBCINST_PROMPTTYPE_COMBOBOX	3
-#define	ODBCINST_PROMPTTYPE_FILENAME	4
-#define	ODBCINST_PROMPTTYPE_HIDDEN	    5
-
-typedef struct tODBCINSTPROPERTY
-{
-	/** pointer to next property, NULL if last property */
-	struct tODBCINSTPROPERTY *pNext;
-
-	/** property name */
-	char szName[INI_MAX_PROPERTY_NAME + 1];
-
-	/** property value */
-	char szValue[INI_MAX_PROPERTY_VALUE + 1];
-
-	/** PROMPTTYPE_TEXTEDIT, PROMPTTYPE_LISTBOX, PROMPTTYPE_COMBOBOX, PROMPTTYPE_FILENAME */
-	int nPromptType;
-
-	/** array of pointers terminated with a NULL value in array */
-	char **aPromptData;
-
-	/** help on this property (driver setups should keep it short) */
-	char *pszHelp;
-
-	/** CALLER CAN STORE A POINTER TO ? HERE */
-	void *pWidget;
-
-	/** app should refresh widget ie Driver Setup has changed aPromptData or szValue */
-	int bRefresh;
-
-	/** for odbcinst internal use... only first property has valid one */
-	void *hDLL;
-}
-ODBCINSTPROPERTY, *HODBCINSTPROPERTY;
-
-/* 
- * End BIG Hack.
- */
-
-int ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty);
-
-static const char *const aTDSver[] = {
-	"",
-	"4.2",
-	"5.0",
-	"7.0",
-	"7.1",
-	"7.2",
-	"7.3",
-	"7.4",
-	NULL
-};
-
-static const char *const aLanguage[] = {
-	"us_english",
-	NULL
-};
-
-static const char *const aEncryption[] = {
-	TDS_STR_ENCRYPTION_OFF,
-	TDS_STR_ENCRYPTION_REQUEST,
-	TDS_STR_ENCRYPTION_REQUIRE,
-	NULL
-};
-
-static const char *const aBoolean[] = {
-	"Yes",
-	"No",
-	NULL
-};
-
-/*
-static const char *aAuth[] = {
-	"Server",
-	"Domain",
-	"Both",
-	NULL
-};
-*/
-
-static HODBCINSTPROPERTY
-addProperty(HODBCINSTPROPERTY hLastProperty)
-{
-	hLastProperty->pNext = (HODBCINSTPROPERTY) calloc(1, sizeof(ODBCINSTPROPERTY));
-	hLastProperty = hLastProperty->pNext;
-	return hLastProperty;
-}
-
-static HODBCINSTPROPERTY
-definePropertyString(HODBCINSTPROPERTY hLastProperty, const char *name, const char *value, const char *comment)
-{
-	hLastProperty = addProperty(hLastProperty);
-	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
-	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
-	hLastProperty->pszHelp = (char *) strdup(comment);
-	return hLastProperty;
-}
-
-static HODBCINSTPROPERTY
-definePropertyBoolean(HODBCINSTPROPERTY hLastProperty, const char *name, const char *value, const char *comment)
-{
-	hLastProperty = addProperty(hLastProperty);
-	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
-	hLastProperty->aPromptData = malloc(sizeof(aBoolean));
-	memcpy(hLastProperty->aPromptData, aBoolean, sizeof(aBoolean));
-	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
-	hLastProperty->pszHelp = (char *) strdup(comment);
-	return hLastProperty;
-}
-
-static HODBCINSTPROPERTY
-definePropertyHidden(HODBCINSTPROPERTY hLastProperty, const char *name, const char *value, const char *comment)
-{
-	hLastProperty = addProperty(hLastProperty);
-	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_HIDDEN;
-	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
-	hLastProperty->pszHelp = (char *) strdup(comment);
-	return hLastProperty;
-}
-
-static HODBCINSTPROPERTY
-definePropertyList(HODBCINSTPROPERTY hLastProperty, const char *name, const char *value, const void *list, int size, const char *comment)
-{
-	hLastProperty = addProperty(hLastProperty);
-	hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
-	hLastProperty->aPromptData = malloc(size);
-	memcpy(hLastProperty->aPromptData, list, size);
-	strlcpy(hLastProperty->szName, name, INI_MAX_PROPERTY_NAME);
-	strlcpy(hLastProperty->szValue, value, INI_MAX_PROPERTY_VALUE);
-	hLastProperty->pszHelp = (char *) strdup(comment);
-	return hLastProperty;
-}
-
-int
-ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
-{
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Servername, "", 
-		"Name of FreeTDS connection to connect to.\n"
-		"This server name refer to entry in freetds.conf file, not real server name.\n"
-		"This property cannot be used with Server property.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Server, "", 
-		"Name of server to connect to.\n"
-		"This should be the name of real server.\n"
-		"This property cannot be used with Servername property.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Address, "", 
-		"The hostname or ip address of the server.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Port, "1433", 
-		"TCP/IP Port to connect to.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Database, "", 
-		"Default database.");
-
-	hLastProperty = definePropertyList(hLastProperty, odbc_param_TDS_Version, "4.2", (void*) aTDSver, sizeof(aTDSver),
-		"The TDS protocol version.\n"
-		" 4.2 MSSQL 6.5 or Sybase < 10.x\n"
-		" 5.0 Sybase >= 10.x\n"
-		" 7.0 MSSQL 7\n"
-		" 7.1 MSSQL 2000\n"
-		" 7.2 MSSQL 2005\n"
-		" 7.3 MSSQL 2008\n"
-		" 7.4 MSSQL 2012 or 2014"
-		);
-
-	hLastProperty = definePropertyList(hLastProperty, odbc_param_Language, "us_english", (void*) aLanguage, sizeof(aLanguage),
-		"The default language setting.");
-
-	hLastProperty = definePropertyHidden(hLastProperty, odbc_param_TextSize, "", 
-		"Text datatype limit.");
-
-	/* ??? in odbc.ini ??? */
-/*
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_UID, "", 
-		"User ID (Beware of security issues).");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_PWD, "", 
-		"Password (Beware of security issues).");
-*/
-
-/*
-	hLastProperty = definePropertyList(hLastProperty, odbc_param_Authentication, "Server", aAuth, sizeof(aAuth),
-		"The server authentication mechanism.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Domain, "", 
-		"The default domain to use when using Domain Authentication.");
-*/
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_PacketSize, "", 
-		"Size of network packets.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_ClientCharset, "", 
-		"The client character set name to convert application characters to UCS-2 in TDS 7.0 and higher.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_DumpFile, "",
-		"Specifies the location of a tds dump file and turns on logging.");
-
-	hLastProperty = definePropertyBoolean(hLastProperty, odbc_param_DumpFileAppend, "",
-		"Appends dump file instead of overwriting it. Useful for debugging when many processes are active.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_DebugFlags, "", 
-		"Sets granularity of logging. A set of bit that specify levels and informations. See table below for bit specification.");
-
-	hLastProperty = definePropertyList(hLastProperty, odbc_param_Encryption, TDS_STR_ENCRYPTION_OFF, aEncryption, sizeof(aEncryption),
-		"The encryption method.");
-
-	hLastProperty = definePropertyString(hLastProperty, odbc_param_Timeout, "10",
-		"Connection timeout.");
-
-	return 1;
-}
-
-#endif
