@@ -35,6 +35,7 @@
 #include <freetds/utils/string.h>
 #include <freetds/convert.h>
 #include <freetds/enum_cap.h>
+#include <freetds/utils/bjoern-utf8.h>
 #include <odbcss.h>
 
 /**
@@ -54,7 +55,7 @@ static DSTR *odbc_mb2utf(TDS_DBC *dbc, DSTR *res, const char *s, unsigned int le
 static DSTR *odbc_wide2utf(DSTR *res, const SQLWCHAR *s, unsigned int len);
 #endif
 
-int
+SQLRETURN
 odbc_set_stmt_query(TDS_STMT * stmt, const ODBC_CHAR *sql, int sql_len _WIDE)
 {
 	if (sql_len == SQL_NTS)
@@ -74,10 +75,11 @@ odbc_set_stmt_query(TDS_STMT * stmt, const ODBC_CHAR *sql, int sql_len _WIDE)
 	stmt->is_prepared_query = 0;
 	stmt->prepared_query_is_func = 0;
 	stmt->prepared_query_is_rpc = 0;
-	stmt->prepared_pos = NULL;
+	stmt->prepared_pos = 0;
 	stmt->curr_param_row = 0;
 	stmt->num_param_rows = 1;
 	stmt->need_reprepare = 0;
+	stmt->params_queried = 0;
 
 	if (!odbc_dstr_copy(stmt->dbc, &stmt->query, sql_len, sql))
 		return SQL_ERROR;
@@ -127,7 +129,7 @@ odbc_iso2utf(DSTR *res, const char *s, unsigned int len)
 			*p++ = u;
 		}
 	}
-	assert(p+1-out <= o_len);
+	assert(p + 1 <= out + o_len);
 	return tds_dstr_setlen(res, p - out);
 }
 
@@ -210,7 +212,7 @@ odbc_wide2utf(DSTR *res, const SQLWCHAR *s, unsigned int len)
 		}
 		*p++ = 0x80 | (0x3f & u);
 	}
-	assert(p+1-out <= o_len);
+	assert(p + 1 <= out + o_len);
 	return tds_dstr_setlen(res, p - out);
 }
 
@@ -253,6 +255,16 @@ odbc_mb2utf(TDS_DBC *dbc, DSTR *res, const char *s, unsigned int len)
 #endif
 
 #ifdef ENABLE_ODBC_WIDE
+/**
+ * Copy a string from client setting according to ODBC convenction
+ * @param dbc       database connection. Can't be NULL
+ * @param s         output string
+ * @param size      size of str, Can be SQL_NTS
+ * @param str       string to convert
+ * @param flag      set of flags.
+ *                  0x01 wide string in buffer
+ *                  0x20 size is in bytes, not characters
+ */
 DSTR*
 odbc_dstr_copy_flag(TDS_DBC *dbc, DSTR *s, int size, const ODBC_CHAR * str, int flag)
 {
@@ -281,7 +293,10 @@ odbc_dstr_copy(TDS_DBC *dbc, DSTR *s, int size, const ODBC_CHAR * str)
  * @param pcbBuffer pointer to SQLSMALLINT to hold string size
  * @param s         string to copy
  * @param len       len of string to copy. <0 null terminated
- * @param flag      set of flag 0x10 SQLINTEGER
+ * @param flag      set of flags.
+ *                  0x01 wide string in buffer
+ *                  0x10 pcbBuffer is SQLINTEGER otherwise SQLSMALLINT
+ *                  0x20 size is in bytes, not characters
  */
 SQLRETURN
 odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void FAR * pcbBuffer, const char *s, int len, int flag)
@@ -302,6 +317,7 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 	if ((flag & 1) != 0) {
 		/* wide characters */
 		const unsigned char *p = (const unsigned char*) s;
+		const unsigned char *const p_end = p + len;
 		SQLWCHAR *dest = (SQLWCHAR*) buffer;
 
 		if (flag&0x20)
@@ -309,30 +325,14 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 #ifndef NDEBUG
 		initial_size = cbBuffer;
 #endif
-		while (len) {
-			unsigned char mask;
-			unsigned u;
-			int l;
+		while (p < p_end) {
+			uint32_t u, state = UTF8_ACCEPT;
 
-			if (!(p[0] & 0x80)) {
-				mask = 0x7f; l = 1;
-			} else if ((p[0] & 0xe0) == 0xc0) {
-				mask = 0x1f; l = 2;
-			} else if ((p[0] & 0xf0) == 0xe0) {
-				mask = 0x0f; l = 3;
-			} else if ((p[0] & 0xf8) == 0xf0) {
-				mask = 0x07; l = 4;
-			} else if ((p[0] & 0xfc) == 0xf8) {
-				mask = 0x03; l = 5;
-			} else {
-				mask = 0x7f; l = 1;
-			}
-			if (len < l)
+			while (decode_utf8(&state, &u, *p++) > UTF8_REJECT && p < p_end)
+				continue;
+			if (state != UTF8_ACCEPT)
 				break;
-			len -= l;
-			u = *p++ & mask;
-			while(--l)
-				u = (u << 6) | (*p++ & 0x3f);
+
 			++out_len;
 			if (SIZEOF_SQLWCHAR == 2 && u >= 0x10000 && u < 0x110000u)
 				++out_len;
@@ -369,35 +369,20 @@ odbc_set_string_flag(TDS_DBC *dbc, SQLPOINTER buffer, SQLINTEGER cbBuffer, void
 	} else if (!dbc || !dbc->mb_conv) {
 		/* to ISO-8859-1 */
 		const unsigned char *p = (const unsigned char*) s;
+		const unsigned char *const p_end = p + len;
 		unsigned char *dest = (unsigned char*) buffer;
 
 #ifndef NDEBUG
 		initial_size = cbBuffer;
 #endif
-		while (len) {
-			unsigned char mask;
-			unsigned u;
-			int l;
+		while (p < p_end) {
+			uint32_t u, state = UTF8_ACCEPT;
 
-			if (!(p[0] & 0x80)) {
-				mask = 0x7f; l = 1;
-			} else if ((p[0] & 0xe0) == 0xc0) {
-				mask = 0x1f; l = 2;
-			} else if ((p[0] & 0xf0) == 0xe0) {
-				mask = 0x0f; l = 3;
-			} else if ((p[0] & 0xf8) == 0xf0) {
-				mask = 0x07; l = 4;
-			} else if ((p[0] & 0xfc) == 0xf8) {
-				mask = 0x03; l = 5;
-			} else {
-				mask = 0x7f; l = 1;
-			}
-			if (len < l)
+			while (decode_utf8(&state, &u, *p++) > UTF8_REJECT && p < p_end)
+				continue;
+			if (state != UTF8_ACCEPT)
 				break;
-			len -= l;
-			u = *p++ & mask;
-			while(--l)
-				u = (u << 6) | (*p++ & 0x3f);
+
 			++out_len;
 			if (!dest)
 				continue;
@@ -654,6 +639,7 @@ odbc_c_to_server_type(int c_type)
 	case SQL_C_SSHORT:
 		return SYBINT2;
 	case SQL_C_STINYINT:
+		return SYBSINT1;
 	case SQL_C_TINYINT:
 	case SQL_C_UTINYINT:
 		return SYBINT1;
@@ -740,6 +726,8 @@ odbc_sql_to_c_type_default(int sql_type)
 	case SQL_VARBINARY:
 	case SQL_LONGVARBINARY:
 		return SQL_C_BINARY;
+	case SQL_SS_TABLE:
+		return SQL_C_BINARY;
 		/* TODO interval types */
 	default:
 		return 0;
@@ -857,6 +845,8 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 		return SYBVARBINARY;
 	case SQL_LONGVARBINARY:
 		return SYBIMAGE;
+	case SQL_SS_TABLE:
+		return SYBMSTABLE;
 		/* TODO interval types */
 	default:
 		return TDS_INVALID_TYPE;
@@ -956,6 +946,8 @@ odbc_get_param_len(const struct _drecord *drec_axd, const struct _drecord *drec_
 	TYPE_NORMAL(SQL_SS_VARIANT) \
 	TYPE_NORMAL(SQL_TYPE_DATE) \
 	TYPE_NORMAL(SQL_TYPE_TIME) \
+\
+	TYPE_NORMAL(SQL_SS_TABLE) \
 \
 	TYPE_VERBOSE_START(SQL_DATETIME) \
 	TYPE_VERBOSE_DATE(SQL_DATETIME, SQL_CODE_TIMESTAMP, SQL_TYPE_TIMESTAMP, SQL_TIMESTAMP) \
@@ -970,7 +962,8 @@ odbc_get_concise_sql_type(SQLSMALLINT type, SQLSMALLINT interval)
 #define TYPE_VERBOSE_DATE(t, interval, concise, old) \
 	case interval: return concise;
 #define TYPE_VERBOSE_END(t) \
-	}
+	} \
+	break;
 
 	switch (type) {
 		SQL_TYPES;
@@ -1006,9 +999,13 @@ odbc_set_concise_sql_type(SQLSMALLINT concise_type, struct _drecord * drec, int
 		return SQL_ERROR;
 	}
 	if (!check_only) {
+		if (drec->sql_desc_concise_type == SQL_SS_TABLE)
+			tvp_free((SQLTVP *) drec->sql_desc_data_ptr);
+
 		drec->sql_desc_concise_type = concise_type;
 		drec->sql_desc_type = type;
 		drec->sql_desc_datetime_interval_code = interval_code;
+		drec->sql_desc_data_ptr = NULL;
 
 		switch (drec->sql_desc_type) {
 		case SQL_NUMERIC:
@@ -1091,7 +1088,8 @@ odbc_get_concise_c_type(SQLSMALLINT type, SQLSMALLINT interval)
 #define TYPE_VERBOSE_DATE(t, interval, concise, old) \
 	case interval: return concise;
 #define TYPE_VERBOSE_END(t) \
-	}
+	} \
+	break;
 
 	switch (type) {
 		C_TYPES;
@@ -1133,6 +1131,7 @@ odbc_set_concise_c_type(SQLSMALLINT concise_type, struct _drecord * drec, int ch
 		drec->sql_desc_concise_type = concise_type;
 		drec->sql_desc_type = type;
 		drec->sql_desc_datetime_interval_code = interval_code;
+		drec->sql_desc_data_ptr = NULL;
 
 		switch (drec->sql_desc_type) {
 		case SQL_C_NUMERIC:
