@@ -63,7 +63,7 @@ typedef struct tds_pbcb
 	/** buffer length */
 	unsigned int cb;
 	/** true is buffer came from malloc */
-	unsigned int from_malloc;
+	bool from_malloc;
 } TDSPBCB;
 
 static TDSRET tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo);
@@ -111,8 +111,7 @@ tds_bcp_init(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS))
 		   == TDS_SUCCESS)
 		continue;
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(rc);
 
 	/* copy the results info from the TDS socket */
 	if (!tds->res_info)
@@ -163,7 +162,17 @@ tds_bcp_init(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		curcol->column_computed = resinfo->columns[i]->column_computed;
 		
 		memcpy(curcol->column_collation, resinfo->columns[i]->column_collation, 5);
-		
+
+		/* From MS documentation:
+		 * Note that for INSERT BULK operations, XMLTYPE is to be sent as NVARCHAR(N) or NVARCHAR(MAX)
+		 * data type. An error is produced if XMLTYPE is specified.
+		 */
+		if (curcol->on_server.column_type == SYBMSXML) {
+			curcol->on_server.column_type = XSYBNVARCHAR;
+			curcol->column_type = SYBVARCHAR;
+			memcpy(curcol->column_collation, tds->conn->collation, 5);
+		}
+
 		if (is_numeric_type(curcol->column_type)) {
 			curcol->bcp_column_data = tds_alloc_bcp_column_data(sizeof(TDS_NUMERIC));
 			((TDS_NUMERIC *) curcol->bcp_column_data->data)->precision = curcol->column_prec;
@@ -243,7 +252,7 @@ tds7_build_bulk_insert_stmt(TDSSOCKET * tds, TDSPBCB * clause, TDSCOLUMN * bcpco
 		strcpy(temp, clause->pb);
 		if (clause->from_malloc)
 			free(clause->pb);
-		clause->from_malloc = 1;
+		clause->from_malloc = true;
 		clause->pb = temp;
 		clause->cb *= 2;
 	}
@@ -277,7 +286,7 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 
 		colclause.pb = clause_buffer;
 		colclause.cb = sizeof(clause_buffer);
-		colclause.from_malloc = 0;
+		colclause.from_malloc = false;
 
 		/* TODO avoid asprintf, use always malloc-ed buffer */
 		firstcol = 1;
@@ -328,7 +337,8 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 }
 
 static TDSRET
-tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, int offset)
+tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
+		 tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset)
 {
 	int i;
 
@@ -366,6 +376,11 @@ tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_c
 		save_data = bindcol->column_data;
 		assert(bindcol->column_data == NULL);
 		if (bindcol->bcp_column_data->is_null) {
+			if (!bindcol->column_nullable && !is_nullable_type(bindcol->on_server.column_type)) {
+				if (null_error)
+					null_error(bcpinfo, i, offset);
+				return TDS_FAIL;
+			}
 			bindcol->column_cur_size = -1;
 		} else if (is_blob_col(bindcol)) {
 			bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
@@ -380,8 +395,7 @@ tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_c
 		bindcol->column_cur_size = save_size;
 		bindcol->column_data = save_data;
 
-		if (TDS_FAILED(rc))
-			return rc;
+		TDS_PROPAGATE(rc);
 	}
 	return TDS_SUCCESS;
 }
@@ -435,12 +449,10 @@ tds5_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
 		TDSCOLUMN  *bindcol = bcpinfo->bindinfo->columns[i];
 		if (is_blob_type(bindcol->on_server.column_type)) {
-			TDSRET rc = get_col_data(bcpinfo, bindcol, offset);
-			if (TDS_FAILED(rc))
-				return rc;
+			TDS_PROPAGATE(get_col_data(bcpinfo, bindcol, offset));
 			/* unknown but zero */
 			tds_put_smallint(tds, 0);
-			tds_put_byte(tds, bindcol->on_server.column_type);
+			TDS_PUT_BYTE(tds, bindcol->on_server.column_type);
 			tds_put_byte(tds, 0xff - blob_cols);
 			/*
 			 * offset of txptr we stashed during variable
@@ -478,7 +490,7 @@ tds_bcp_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 		return TDS_FAIL;
 
 	if (IS_TDS7_PLUS(tds->conn))
-		rc = tds7_send_record(tds, bcpinfo, get_col_data, offset);
+		rc = tds7_send_record(tds, bcpinfo, get_col_data, null_error, offset);
 	else
 		rc = tds5_send_record(tds, bcpinfo, get_col_data, null_error, offset);
 
@@ -487,7 +499,7 @@ tds_bcp_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
 }
 
 static inline void
-tds5_swap_data(const TDSCOLUMN *col, void *p)
+tds5_swap_data(const TDSCOLUMN *col TDS_UNUSED, void *p TDS_UNUSED)
 {
 #ifdef WORDS_BIGENDIAN
 	tds_swap_datatype(tds_get_conversion_type(col->on_server.column_type, col->column_size), p);
@@ -635,7 +647,7 @@ tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_
 		 * Is this column of "variable" type, i.e. NULLable
 		 * or naturally variable length e.g. VARCHAR
 		 */
-		if (bcpinfo->sybase_count > i) {
+		if (bcpinfo->sybase_count > (TDS_INT) i) {
 			if (bcpinfo->sybase_colinfo[i].offset >= 0)
 				continue;
 		} else {
@@ -704,7 +716,7 @@ tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_
 
 	if (ncols) {
 		TDS_UCHAR *poff = rowbuffer + row_pos;
-		unsigned int pfx_top = offsets[ncols] / 256;
+		unsigned int pfx_top = offsets[ncols] >> 8;
 
 		tdsdump_log(TDS_DBG_FUNC, "ncols=%u poff=%p [%u]\n", ncols, poff, offsets[ncols]);
 
@@ -714,7 +726,7 @@ tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_
 			unsigned int n_pfx = 1;
 
 			for (i = 0; i <= ncols ; ++i)
-				if ((offsets[i] / 256) < pfx_top)
+				if ((offsets[i] >> 8) < pfx_top)
 					++n_pfx;
 			*poff++ = n_pfx;
 			--pfx_top;
@@ -794,7 +806,7 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		else
 			tds_put_smallint(tds, bcpcol->column_usertype);
 		tds_put_smallint(tds, bcpcol->column_flags);
-		tds_put_byte(tds, bcpcol->on_server.column_type);
+		TDS_PUT_BYTE(tds, bcpcol->on_server.column_type);
 
 		assert(bcpcol->funcs);
 		bcpcol->funcs->put_info(tds, bcpcol);
@@ -845,8 +857,6 @@ tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 TDSRET
 tds_bcp_done(TDSSOCKET *tds, int *rows_copied)
 {
-	TDSRET rc;
-
 	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_done(%p, %p)\n", tds, rows_copied);
 
 	if (tds->out_flag != TDS_BULK || tds_set_state(tds, TDS_WRITING) != TDS_WRITING)
@@ -856,9 +866,7 @@ tds_bcp_done(TDSSOCKET *tds, int *rows_copied)
 
 	tds_set_state(tds, TDS_PENDING);
 
-	rc = tds_process_simple_query(tds);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(tds_process_simple_query(tds));
 
 	if (rows_copied)
 		*rows_copied = tds->rows_affected;
@@ -882,9 +890,7 @@ tds_bcp_start(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 	if (!IS_TDS50_PLUS(tds->conn))
 		return TDS_FAIL;
 
-	rc = tds_submit_query(tds, bcpinfo->insert_stmt);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(tds_submit_query(tds, bcpinfo->insert_stmt));
 
 	/* set we want to switch to bulk state */
 	tds->bulk_query = true;
@@ -896,8 +902,7 @@ tds_bcp_start(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 		rc = tds5_process_insert_bulk_reply(tds, bcpinfo);
 	else
 		rc = tds_process_simple_query(tds);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(rc);
 
 	tds->out_flag = TDS_BULK;
 	if (tds_set_state(tds, TDS_SENDING) != TDS_SENDING)
@@ -1013,7 +1018,7 @@ tds5_process_insert_bulk_reply(TDSSOCKET * tds, TDSBCPINFO *bcpinfo)
 				col_flags |= 1 << icol;
 				cols_values[icol] = dres.i;
 			}
-			/* save informations */
+			/* save information */
 			if (col_flags != BULKCOL_ALL ||
 				cols_values[BULKCOL_colcnt] < 1 ||
 				cols_values[BULKCOL_colcnt] > 4096 || /* limit of columns accepted */
@@ -1066,7 +1071,7 @@ tds5_process_insert_bulk_reply(TDSSOCKET * tds, TDSBCPINFO *bcpinfo)
  * Free row data allocated in the result set.
  */
 static void 
-tds_bcp_row_free(TDSRESULTINFO* result, unsigned char *row)
+tds_bcp_row_free(TDSRESULTINFO* result, unsigned char *row TDS_UNUSED)
 {
 	result->row_size = 0;
 	TDS_ZERO_FREE(result->current_row);
@@ -1091,9 +1096,7 @@ tds_bcp_start_copy_in(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 	
 	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_start_copy_in(%p, %p)\n", tds, bcpinfo);
 
-	rc = tds_bcp_start_insert_stmt(tds, bcpinfo);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(tds_bcp_start_insert_stmt(tds, bcpinfo));
 
 	rc = tds_bcp_start(tds, bcpinfo);
 	if (TDS_FAILED(rc)) {
@@ -1133,8 +1136,7 @@ tds_bcp_start_copy_in(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 			if (is_nullable_type(bcpcol->on_server.column_type) || bcpcol->column_nullable) {
 				var_cols++;
 				variable_col_len_tot += column_bcp_data_size;
-			}
-			else {
+			} else {
 				fixed_col_len_tot += column_bcp_data_size;
 			}
 		}
@@ -1283,8 +1285,7 @@ tds_bcp_fread(TDSSOCKET * tds, TDSICONV * char_conv, FILE * stream, const char *
 	funlockfile(stream);
 	free(r.left);
 
-	if (TDS_FAILED(res))
-		return res;
+	TDS_PROPAGATE(res);
 
 	*outbytes = w.size;
 
@@ -1317,16 +1318,13 @@ tds_writetext_start(TDSSOCKET *tds, const char *objname, const char *textptr, co
 	rc = tds_submit_queryf(tds,
 			      "writetext bulk %s 0x%s timestamp = 0x%s%s",
 			      objname, textptr, timestamp, with_log ? " with log" : "");
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(rc);
 
 	/* set we want to switch to bulk state */
 	tds->bulk_query = true;
 
 	/* read the end token */
-	rc = tds_process_simple_query(tds);
-	if (TDS_FAILED(rc))
-		return rc;
+	TDS_PROPAGATE(tds_process_simple_query(tds));
 
 	tds->out_flag = TDS_BULK;
 	if (tds_set_state(tds, TDS_WRITING) != TDS_WRITING)
